#!/usr/bin/env python3

import argparse
import sys
import pyfiglet
import os

from .utility import util
from . import port_scanner_cli

"""
Pentest Tools - Lightweight Pentest Toolkit

Note:
  - Only IPv4 addresses are supported.
  - Tool types must be valid and recognized by the toolkit.
  - Port scanner just for scanning ports (not including host scanning)
"""

def main():
    """
    Process all arguments, invoke specific module to achieve different functionality
    """
    banner = pyfiglet.figlet_format("Pentest Tools")
    print(banner)

    parser = argparse.ArgumentParser(description="Lightweight Pentest Toolkit")
    parser.add_argument("ip", help="Target IP address")
    parser.add_argument("-t", "--type", help="Tools refer to use. E.g. PS(for port scanner)")
    parser.add_argument("-p", "--ports", help="Port(s) to scan. E.g. 22,80 or 1-1000")
    parser.add_argument("-PU", action="store_true", help="Use UDP ping for port scan")
    parser.add_argument("-PS", action="store_true", help="Use TCP SYN packet for port scan. This is the default option")
    parser.add_argument("-PH", action="store_true", help="Hybrid, using udp and tcp scan")
    parser.add_argument("-v", action="store_true", help="Detailed information for query")
    args = parser.parse_args()
    
    print(args)

    if not util.is_valid_ipv4(args.ip):
        print("FormatError: Invalid IPv4 address.")
        sys.exit(1)
    elif not util.is_valid_type(args.type):
        print("FormatError: Invalid type category.")
        sys.exit(1)

    try:
        if args.ports:
            ports = util.parse_ports(args.ports)
        else:
            if args.PU:
                ports = util.load_default_ports(flag="UDP")
            else:
                ports = util.load_default_ports()
    except ValueError as e:
        print(f"FormatError: {e}")
        sys.exit(1)
    
    # Port Scanner Logic
    if args.type is None or args.type == "PS":
        is_root = os.geteuid() == 0
        port_scanner_cli.process(ports=ports, args=args, is_root=is_root)

if __name__ == "__main__":
    main()
